#include <string>
#define X_DISPLAY_MISSING
#include <Imlib2.h>

#include "bitmap.h"

static int FileExists(const char *FileName) {
  FILE *f = NULL;
  f = fopen( FileName, "rb" );
  if (f != NULL ) {
    fclose(f);
    return true;
    }
  return false;
}

cImageloadBitmap::cImageloadBitmap()
{
}

cImageloadBitmap ::~cImageloadBitmap()
{
}


bool cImageloadBitmap::Load(const char *Filename)
{
#ifdef DEBUG_COVER
  printf("coverviewer: bitmap.c : check for image '%s'\n", Filename);
#endif

  if(FileExists(Filename)) {
#ifdef DEBUG_COVER
    printf("coverviewer: bitmap.c : image found\n");
#endif
    return true;
    }
  else {
#ifdef DEBUG_COVER
    printf("coverviewer: bitmap.c : imagefile dont exists\n");
#endif
    esyslog("coverviewer: bitmap.c : imagefile dont exists");
    return false;
  }

}


Imlib_Image image;
Imlib_Context ctx;


void cImageloadBitmap::LoadImage( const char *Filename, int width, int height, bool ratio, int startx, bool isProgressbar)
{
//  Imlib_Image image;

  image = NULL;

#ifdef DEBUG_COVER
  printf("coverviewer: bitmap.c : load image '%s'\n",Filename);
  printf("coverviewer: bitmap.c : values: width=%d height=%d ratio=%d startx=%d isProgressbar=%d\n", width, height, ratio, startx, isProgressbar );
#endif

  newwidth  = 0;
  newheight = 0;

  image = imlib_load_image(Filename);

  if (!image || height <=0 || width <=0) {
#ifdef DEBUG_COVER
    printf("coverviewer: bitmap.c: no image or wrong dimension\n");
#endif
    esyslog("coverviewer: bitmap.c: no image or wrong dimension");
    }
  else {
    ctx = imlib_context_new();
    imlib_context_push(ctx);

    if(isProgressbar) {
      imlib_context_set_image(image);
      realwidth  = imlib_image_get_width();
      realheight = imlib_image_get_height();
      image = imlib_create_cropped_image(startx, 0, width, height);
#ifdef DEBUG_COVER
      printf("coverviewer: bitmap.c: create a progressbar\n");
#endif
      }
    else {
      if(!ratio) {
        imlib_context_set_image(image);
        realwidth  = imlib_image_get_width();
        realheight = imlib_image_get_height();
        image = imlib_create_cropped_scaled_image(0, 0, realwidth, realheight, width, height); 
#ifdef DEBUG_COVER
        printf("coverviewer: bitmap.c: create an image without aspect ratio:\nimlib_image_get_width=%i imlib_image_get_height=%i\n", imlib_image_get_width(), imlib_image_get_height());
#endif
        }
      else {
        imlib_context_set_image(image);
        realwidth  = imlib_image_get_width();
        realheight = imlib_image_get_height();
        int x1 = realwidth;
        int y1 = realheight;
        int x2 = width;
        int y2 = height;
        float stretched = float(x1) * (float(y2) / float(y1));

#ifdef DEBUG_COVER
        printf("coverviewer: bitmap.c: create an image with aspect ratio x1=%i y1=%i x2=%i y2=%i\n", x1 ,y1 ,x2, y2);
#endif
        image = imlib_create_cropped_scaled_image(0, 0, x1, y1, (int)stretched < x2 ? (int)stretched : x2, y2);
      }
    }
    imlib_context_set_image(image);

    newwidth  = imlib_image_get_width();
    newheight = imlib_image_get_height();

#ifdef DEBUG_COVER
    printf("coverviewer: bitmap.c: new values of image: width=%d  height=%d\n", newwidth, newheight);
#endif

  }
}

void cImageloadBitmap::Render(cImage &bmp)
{

#ifdef DEBUG_COVER
  printf("coverviewer: bitmap.c: start to render image\n");
#endif

  for (int ix = 0; ix < newwidth; ++ix) {
    for (int iy = 0; iy < newheight; ++iy) {

      Imlib_Color iml_col;
      imlib_image_query_pixel(ix,iy,&iml_col);

//      int a;
//      int r;
//      int g;
//      int b;
//      printf("a=%i\n",a);
//      printf("r=%i\n",r);
//      printf("g=%i\n",g);
//      printf("b=%i\n",b);

      tColor col = (uint8_t)iml_col.alpha << 24 | (uint8_t)iml_col.red << 16 | (uint8_t)iml_col.green << 8 | (uint8_t)iml_col.blue;

      bmp.SetPixel(cPoint(ix, iy), col);

    }
  }

#ifdef DEBUG_COVER
  printf("coverviewer: bitmap.c: image rendered !\n");
#endif

#ifdef DEBUG_COVER
  printf("coverviewer: bitmap.c: imlib_free_image()\n");
#endif
  if(image) imlib_free_image();


#ifdef DEBUG_COVER
  printf("coverviewer: bitmap.c: imlib_context_free()\n");
#endif
  if(ctx) imlib_context_free(ctx);

}
