/*
* Image plugin to VDR (C++)
*
* (C) 2004-2005 Andreas Brachold <anbr at users.berlios.de>
* based on (C) 2003 Kai Tobias Burwieck <kai -at- burwieck.net>
*
* This code is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This code is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
* Or, point your browser to http://www.gnu.org/copyleft/gpl.html
*/

#include <time.h>
#include <vdr/plugin.h>
#include <vdr/device.h>

#include "control-image.h"
#include "data-image.h"
#include "setup-image.h"
#include "config.h"
// #include "i18n.h"
#include "list.h"
#include "bitmap.h"
#include "parselog.h"



// ------------ cCVCommands ---------------------------------------------------------------------------------------------- //

cCoverMenu::cCoverMenu(const char *c_file)
:cOsdMenu(tr("Coverviewer: Commands"),2)
{

  d_file = c_file;

  Add(new cOsdItem(hk(tr("Save cover for an album"))));
  Add(new cOsdItem(hk(tr("Save cover for a track"))));
  Add(new cOsdItem(hk(tr("Save cover for an artist"))));

  SetHelp();

  Display();
}

cCoverMenu::~cCoverMenu()
{
}

void cCoverMenu::SetHelp() {
  cOsdMenu::SetHelp(tr("Parent"),NULL,NULL,NULL);
}

void cCoverMenu::CopyAlbum(void)
{
//  char *cmd;
  std::string cmd("");
  std::string datei;
  datei = CoverLog.log_basedir + "cover.jpg"; 

  cmd = "cp -f \"" +d_file+ "\"  \"" +datei+ "\"";

//  asprintf(&cmd, "cp -f \"%s\"  \"%s\"", d_file.c_str(), datei.c_str());
  dsyslog("coverviewer: save as Album :Executing '%s'", cmd.c_str());
  system(cmd.c_str());
//  free(cmd); 
}

void cCoverMenu::CopyTrack(void)
{
//  char *cmd;
  std::string cmd("");
  std::string datei;
  datei = CoverLog.log_filename + "jpg";

  cmd = "cp -f \"" +d_file+ "\"  \"" +datei+ "\"";

//  asprintf(&cmd, "cp -f \"%s\"  \"%s\"", d_file.c_str(), datei.c_str());
  dsyslog("coverviewer: save as Trackname: Executing '%s'", cmd.c_str());
  system(cmd.c_str());
//  free(cmd); 
}

void cCoverMenu::CopyArtist(void)
{
//  char *cmd;
  std::string cmd("");
  std::string datei;
  datei = CoverLog.log_coverdir + "/" + CoverLog.log_artist + ".jpg";

  cmd = "cp -f \"" +d_file+ "\"  \"" +datei+ "\"";

//  asprintf(&cmd, "cp -f \"%s\"  \"%s\"", d_file.c_str(), datei.c_str());
  dsyslog("coverviewer: save as Artist: Executing '%s'", cmd.c_str());
  system(cmd.c_str());
//  free(cmd); 
}


eOSState cCoverMenu::CopyCover(void) {

  int ItemIndex = Current();

  if(ItemIndex == 0)      {
    CopyAlbum();
    }
  else if(ItemIndex == 1) {
    CopyTrack();
    }
  else if(ItemIndex == 2) {
    CopyArtist();
  }

  return osBack;

}


eOSState cCoverMenu::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if(state==osUnknown) {
     switch(Key) {

       case kOk:
             state = CopyCover();
             break;

       case kBlue:
       case kGreen:
       case kYellow:
       case kRed:
             return osBack;

       default:
             return osContinue;;
       }
     }
  return state;
}





//////////////////////////////////////////////////////////////////////////////
/** C-tor Create for control-object to control the image-player
*/
cCoverControl::cCoverControl()
:cOsdObject()
,covermenu(NULL)
{
  m_tStarted = time(NULL);
  m_bSlideShowActiv = ImageSetup.m_bSlideShow;

  picturesize = 0;
  newheight= 0;
  newwidth = 0;  
  rotate = 0;
  ImageSetup.rotate = 0;

  visible=false;
  CanLoadCover=false;
  osd = NULL;

  Layer0Pixmap = NULL;
  Layer1Pixmap = NULL;
  Layer2Pixmap = NULL;
  Layer3Pixmap = NULL;

}


void cCoverControl::SetSlideShow(cSlideShow * pNewSlideShow)
{
  theSlideShow.Assign(pNewSlideShow);
}

//////////////////////////////////////////////////////////////////////////////
/** D-tor Destroy control-object to control the image-player
*/
cCoverControl::~cCoverControl()
{
  // Free OSD Data
  theSlideShow.Shutdown();
  if(covermenu) { delete covermenu; covermenu=NULL; }

  Layer0Pixmap = NULL;
  Layer1Pixmap = NULL;
  Layer2Pixmap = NULL;
  Layer3Pixmap = NULL;

  Hide();
}

void cCoverControl::Hide()
{
  if(osd) { delete osd; osd=NULL; }
  visible = false;
}

//////////////////////////////////////////////////////////////////////////////
/** LOAD PICTURE
*/
void cCoverControl::LoadPicture(void)
{
 
  cImageloadBitmap osdbitmap;
  if(osdbitmap.Load(FileName())) {
    osdbitmap.LoadImage(FileName(), w, h, true, 0, false);
    newwidth  = osdbitmap.newWidth();
    newheight = osdbitmap.newHeight();
    cImage img(cSize(newwidth, newheight));
    osdbitmap.Render(img);
    int posx  = ( w - newwidth)  / 2;
    int posy = ( h - newheight) / 2;

#ifdef BUGHUNT
    printf("width  : %i\n",newwidth);
    printf("height : %i\n",newheight);
    printf("posx   : %i\n",posx);
    printf("posy   : %i\n",posy);
#endif


#ifdef BUGHUNT
    printf("Normal : DrawImage on Layer 1\n");
#endif
    Layer1Pixmap->Clear();
    Layer1Pixmap->DrawImage(cPoint(posx, posy), img);

    Layer3Pixmap->Clear();
    cString buf = cString::sprintf("%d x %d", osdbitmap.realWidth(), osdbitmap.realHeight());
    Layer3Pixmap->DrawText(cPoint(20, 20), buf, clrWhite, clrBlack, cFont::GetFont(fontSml));
  }
}

//////////////////////////////////////////////////////////////////////////////
/** BUILD AREAS
*/
bool cCoverControl::BuildAreas(void)
{
  int res;
  
  res = false;

  if (FileName()== NULL ) {
    esyslog("coverviewer : Oops can't load image : %s\n",FileName());
    }
  else {
    tArea Area[] = {{ 0, 0, w -1, h -1, depth}, };
    eOsdError result = osd->CanHandleAreas(Area, sizeof(Area) / sizeof(Area));
    if (result == oeOk) {
      osd->SetAreas(Area, sizeof(Area) / sizeof(Area));
      res = true;
      }
    else {
      const char *errormsg = NULL;
      switch (result) {
        case oeTooManyAreas:
            errormsg = "Too many OSD areas"; break;
        case oeTooManyColors:
            errormsg = "Too many colors"; break;
        case oeBppNotSupported:
            errormsg = "Depth not supported"; break;
        case oeAreasOverlap:
            errormsg = "Areas are overlapped"; break;
        case oeWrongAlignment:
            errormsg = "Areas not correctly aligned"; break;
        case oeOutOfMemory:
            errormsg = "OSD memory overflow"; break;
        case oeUnknown:
            errormsg = "Unknown OSD error"; break;
        default:
            break;
      } 	

      esyslog("coverviewer: ERROR! OSD open failed! Can't handle areas (%d)-%s\n", result, errormsg);
      if(osd) { Hide(); }
    }
  }
  return res;
}

//////////////////////////////////////////////////////////////////////////////
/** BUILD PIXMAPS 
*/
void cCoverControl::BuildDefault(void)
{
#ifdef BUGHUNT
  printf("Build default\n");
#endif

  Layer0Pixmap = NULL;
  Layer1Pixmap = NULL;
  Layer2Pixmap = NULL;
  Layer3Pixmap = NULL;
  
  if (!Layer0Pixmap) {
#ifdef BUGHUNT
    printf("Layer0Pixmap not exists: Creating now\n");
#endif
    Layer0Pixmap = osd->CreatePixmap(0, cRect(0, 0, w , h ));
//  Layer0Pixmap->Clear();
    Layer0Pixmap->Fill(clrBlack);
  }


  if (!Layer1Pixmap) {
#ifdef BUGHUNT
    printf("Layer1Pixmap not exists: Creating now\n");
#endif
    Layer1Pixmap = osd->CreatePixmap(1, cRect(0, 0, w , h ));
    Layer1Pixmap->Clear();
  }

  if (!Layer2Pixmap) {
#ifdef BUGHUNT
    printf("Layer2Pixmap not exists: Creating now\n");
#endif
    Layer2Pixmap = osd->CreatePixmap(2, cRect(0, 0, w , h ));
    Layer2Pixmap->Clear();
  }


  if (!Layer3Pixmap) {
#ifdef BUGHUNT
    printf("Layer3Pixmap not exists: Creating now\n");
#endif
    Layer3Pixmap = osd->CreatePixmap(3, cRect(0, 0, w , h ));
    Layer3Pixmap->Clear();
  }


#ifdef BUGHUNT
  printf("Default built\n");
#endif


}


//////////////////////////////////////////////////////////////////////////////
/** SHOW
*/
void cCoverControl::Show()
{
  if(FileName()==NULL) {
    esyslog("coverviewer: OOPS ...cant laod coverpicture (NULL)");
    }
  else {

    picturesize=ImageSetup.PictureSize;

    CanLoadCover=true;

    if (!osd) {

      w=640;
      h=480;
      depth=8;

      switch(picturesize) {
       case 0:
	    w = 640;
	    h = 480;
	    depth = 32;
	  break;  
	  case 1:  
	    w = 768;
	    h = 576;
	    depth = 32;
	  break;
	  case 2:
	    w = 1280;
	    h = 720;
	    depth = 32;
	  break;
	  case 3:
	    w = 1920;
	    h = 1080;
	    depth = 32;
	  break;
	  case 4:
	    w = ImageSetup.max_osdwidth;
	    h = ImageSetup.max_osdheight;
	    depth = 32;
	  break;
      }

      osd=cOsdProvider::NewOsd(ImageSetup.OffsetLeft, ImageSetup.OffsetTop, 19);

      if (BuildAreas())
        BuildDefault();
      else {
        esyslog("coverviewer: ERROR: Cannot build areas -> ABORT");
        return;
      }
    }

    if(CanLoadCover) {
#ifdef BUGHUNT
      Start = cTimeMs::Now();
#endif
#ifdef BUGHUNT
      printf("LoadPicture\n");
#endif
      //Draw image
      LoadPicture();
      CanLoadCover = false;
      m_tStarted = time(NULL);

#ifdef BUGHUNT
      printf("None effect\n");
#endif
      Flush();
    }
    ImageSetup.rotate = 0;
  }
}




void cCoverControl::Flush(void)
{
  if(osd) osd->Flush();
}  


//////////////////////////////////////////////////////////////////////////////
/** VDR-Callback entry for Key was processed
@return eOSState
@param eKeys Key - the processed Keycode
*/
eOSState cCoverControl::ProcessKey(eKeys Key)
{

  SlideImage();

  if(covermenu) {
    eOSState eOSRet = covermenu->ProcessKey(Key);
    
    switch(eOSRet) {
      case osBack:
         delete covermenu;
	 covermenu = NULL;
         visible =false;
         Show();

         return osContinue;
      default: return eOSRet;
    }
    
  }    
  else {
 
    // check for next image
    SlideImage();

    eOSState state = cOsdObject::ProcessKey(Key);

    if(state==osUnknown) {
      switch(Key) {

        //processed global keybindings
        // Toggle between Play/Pause
        case kPlay:     
        case kPause:   ToogleSlideShowActiv();
                       break;

        // Stop Plugin    
        case kBlue:
        case kRed: 
        case kBack:
        case kStop:     return osEnd;

  
      
         // Change time how long image is see
         case k1|k_Repeat:
         case k1:       DecSlideTime();
                        break;

         case k3|k_Repeat:
         case k3:     
                        IncSlideTime(); 
                        break;
  
         // Navigate between images
         case kLeft:
         case kGreen:
	                if( m_bSlideShowActiv && ImageTotal() >0) {
		          PrevImage(1);
		          CanLoadCover = true;
		          Show();
		        }
		        break;

         case kRight:
         case kYellow:
	                if( m_bSlideShowActiv && ImageTotal() >0) {
		          NextImage(1);
		          CanLoadCover = true;
		          Show();
		        }
		        break;
 
         case kOk: 
		        m_file = FileName();
		        CoverLog.ParseLog();
                        Hide();
                        covermenu = new cCoverMenu(m_file.c_str());
		        break;

         case kDown:
	                if( m_bSlideShowActiv && ImageTotal() >0) {
		          PrevImage(3);	
//                rotate =1;

		          CanLoadCover = true;
		          Show();
		        }
		        break;

         case kUp:      if( m_bSlideShowActiv && ImageTotal() >0) {
		          NextImage(3);
//                rotate =0;
		          CanLoadCover = true;
		          Show();
		        }
		        break;

         default: break;
      }

    state = osContinue;
    }

  return state;
  }


}


//////////////////////////////////////////////////////////////////////////////
/** Toogle between Play and Stop of the current SlideShow 
*/
void cCoverControl::ToogleSlideShowActiv(void)
{
  m_bSlideShowActiv = !m_bSlideShowActiv;
}


//////////////////////////////////////////////////////////////////////////////
/** Check to get access for to viewed file
@return bool - false access was denied
*/
bool cCoverControl::CheckAccess() const
{
  const char *szFileName = FileName();

  if(szFileName 
    && 0 == access(szFileName,F_OK))
  {
    return true;
  }        

  return false;
}


void cCoverControl::OriginalImage(bool bCached)
{
  if(!CheckAccess())
  {
    esyslog("coverviewer: Operation failed");
  }
}



void cCoverControl::NextImage(int Step)
{
  theSlideShow.NextImage(Step);
  OriginalImage(true);
}

void cCoverControl::PrevImage(int Step)
{
  theSlideShow.PrevImage(Step);
  OriginalImage(true);
}


void cCoverControl::GotoImage(unsigned int Pict)
{
  theSlideShow.GotoImage(Pict);
  OriginalImage(true);
}


int cCoverControl::ImageTotal(void) const
{
  return theSlideShow.ImageTotal();
}


int cCoverControl::ImageCurrent(void) const
{
  return theSlideShow.ImageCurrent();
}


const char* cCoverControl::FileName(void) const
{
  std::string datei;
  datei = configdir;
  datei = datei + "/moronsuite/coverviewer/images/splash.png";

  if(ImageTotal() > 0) {
    cPicture* pImage = theSlideShow.GetImage();
    return pImage?pImage->Name():NULL;
    }
  else {
    return datei.c_str();
  }
}


void cCoverControl::SlideImage()
{
  if( m_bSlideShowActiv && ImageTotal() >0 )
  {
    if(ImageSetup.m_nSSsec <= 
        (time(NULL) - m_tStarted))
    {
      CanLoadCover = true;

      NextImage(1);
      Show();
    }
  }

}


void cCoverControl::IncSlideTime(void)
{
  if(ImageSetup.m_nSSsec < cImageSetup::m_cSSMax) {
	    ImageSetup.m_nSSsec++;
	}
}


void cCoverControl::DecSlideTime(void)
{
  if(ImageSetup.m_nSSsec > cImageSetup::m_cSSMin) {
	    ImageSetup.m_nSSsec--;
	}
}
