/*
 * Image plugin to VDR (C++)
 *
 * (C) 2004-2005 Andreas Brachold <anbr at users.berlios.de>
 * based on (C) 2003 Kai Tobias Burwieck <kai -at- burwieck.net>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

// --- cCoverControl ---------------------------------------------------------

#ifndef ___DVB_IMAGE_CONTROL_H
#define ___DVB_IMAGE_CONTROL_H

#include <string>
#include <vdr/osdbase.h>


#define memberof(x) (sizeof(x)/sizeof(*x))

class cImage;
class cSlideShow;


class cCoverMenu : public cOsdMenu {
private:
  eOSState CopyCover(void);
  void SetHelp();
  std::string d_file;
public:
  cCoverMenu(const char *c_file);
  virtual ~cCoverMenu();
  virtual eOSState ProcessKey(eKeys Key);
  void CopyAlbum(void);
  void CopyTrack(void);
  void CopyArtist(void);
};

class cCoverControl : public cOsdObject
{
  /** View our picture in a slideshow */
  bool                    m_bSlideShowActiv; 
  /** date at image was open*/
  time_t                  m_tStarted;
  /** Remember the active Slideshow mode before Jump or Zoom used */
  bool                    m_bSlideShowBefore;

private:  
  cOsd *osd;
  cCoverMenu *covermenu;

  uint64_t Start;

  int w, h, picturesize, alpha, depth, newwidth, newheight, rotate;
  bool visible, flush, CanLoadCover;
  
  cPixmap *Layer0Pixmap;
  cPixmap *Layer1Pixmap;
  cPixmap *Layer2Pixmap;
  cPixmap *Layer3Pixmap;

  std::string m_file;

  void ShowOSD(bool open=false);
  void HideOSD(void);
  void ShowProgress(void);
  /** Deliver the current number of viewed Image */
  int ImageCurrent(void) const;
  /** Deliver the total number of viewed Image */
  int ImageTotal(void) const;
  /** Deliver the filename from the current number of viewed Image */
  const char* FileName() const;
  /** Toogle between Play and Stop of the current SlideShow */
  void ToogleSlideShowActiv(void);
  /** Path of current imagefile */
  char CurrImageFile[255];

  void LoadPicture(void);
  bool BuildAreas(void);
  void BuildDefault(void);
  void NextImage(int Step);
  void PrevImage(int Step);
  void GotoImage(unsigned int Pict);
  void OriginalImage(bool bCached);
  void IncSlideTime(void);
  void DecSlideTime(void);
  bool CheckAccess() const;
  inline void Flush(void);
public:
  cCoverControl(void);
  virtual ~ cCoverControl();
  virtual eOSState ProcessKey(eKeys Key);
  virtual void Show(void);
  virtual void Hide(void);
  static void SetSlideShow(cSlideShow * pNewSlideShow);

protected:  
  void SlideImage();
};

#endif				//___DVB_IMAGE_CONTROL_H
