/*
 * Copyright (C) 2006-2009 Alex Lasnier <alex@fepg.org>
 *
 * This file is part of Radio Info
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "tools.h"


///////////////////////////////////////////////////////////////////////////////


#ifdef RI_DEBUG

#include <stdarg.h>
#include <stdlib.h> 

static uint8_t logType = 0;

void SetLogType(uint8_t type) { logType = type; }

void dprint(uint8_t type, const char* msg, ...)
{
  if (!msg) return;
  
  if (!(type & logType)) return; // no logging for this type
  
  char* output = NULL;
  va_list ap;
  va_start(ap, msg);
  vasprintf(&output, msg, ap);
  va_end(ap);
  
  if (type == L_ERR)  
    fprintf(stderr, "[RadioInfo] \033[31m%s\n\033[0m", output);  
  else 
    fprintf(stderr, "[RadioInfo] %s\n", output);
     
  fflush(stderr);
  free(output);  
}

#endif // RI_DEBUG


//-----------------------------------------------------------------------------

void HexDump(const uint8_t* d, int length)
{
  fprintf(stderr,"      ");
  for (int i=0; i<length; i++) {
    fprintf(stderr,"%02X ", d[i]);
    if (i%16 == 15) fprintf(stderr,"\n      ");
  }
  fprintf(stderr,"\n\n");
}


//-----------------------------------------------------------------------------

const char* OsdErrorToString(eOsdError err)
{
  const char* e = "Unknown";
  switch (err)
  {
    case oeOk:              e = "Ok"; break;
    case oeTooManyAreas:    e = "Too Many Areas";    break;
    case oeTooManyColors:   e = "Too Many Colors";   break;
    case oeBppNotSupported: e = "Bpp Not Supported"; break;
    case oeAreasOverlap:    e = "Areas Overlap";     break;
    case oeWrongAlignment:  e = "Wrong Alignment";   break;
    case oeOutOfMemory:     e = "Out Of Memory";     break;
    case oeWrongAreaSize:   e = "Wrong Area Size";   break;
    case oeUnknown: break;
  }
  return e;
}


///////////////////////////////////////////////////////////////////////////////

