
#include <string>
#include <fstream>
#include <vdr/menu.h>

#include <vdr/osd.h>
#include <vdr/menuitems.h>
#include <vdr/remote.h>

#include "liveticker.h"
#include "sportngtools.h"
#include "setup.h"
// #include "i18n.h"


#define BAR_COLOR		0xA024FC24 /*clrGreen62*/
#define TEXT_COLOR	0xFF000000 /*clrBlack*/
#define EVENT_COLOR	0xFF000000 /*clrBlack*/

bool cLiveEvent::updated4new=false;
cLiveEvent::cLiveEvent(const char *Id, const char *Text)
{
	id=strdup(Id);
	text=strdup(Text);
	updated = updated4new;
	touched = true;
} 
void cLiveEvent::SetText(const char* Text)
{
	touched = true;
	if(strcmp(text, Text))
	{
		updated = true;
		free(text);
		text = strdup(Text);
	}
}


cLiveTicker* cLiveTicker::liveTicker=NULL;
bool cLiveTicker::LiveTickerOsd=false;

cLiveTicker::cLiveTicker() : cThread("SportNG-Liveticker")
{
	if (liveTicker)
	{
		esyslog("%s: attempt to create more than one Liveticker - exiting!", PLUGIN_NAME_I18N);
		exit(2);
	}
	liveTicker = this;
}
cLiveTicker::~cLiveTicker()
{
	liveTicker=NULL;
	Cancel(3);
}
void cLiveTicker::Action(void)
{
//	time_t NextUpdate = GetNextUpdate();
	
	cTimeMs Interval;
	while(Running() && SportNGSetup.liveticker.activ)
	{
		if(SportNGSetup.liveticker.interval!=0 && !Interval.TimedOut())
		{
			cCondWait::SleepMs(500);// Wenn Intervall und noch nicht TimeOut, dann nur warten 
			continue;
		}
		if(SportNGSetup.liveticker.interval==0)
		{
			Lock();	// wenn Update luft mu Anzeige warten
			cLiveEvent *Event;
			for(Event = First(); Event; Event=Next(Event))
				if(Event->Updated()) break;
			Unlock();
			if(Event==NULL)	// Ende der Liste erreicht
			{
				cCondWait::SleepMs(500);// auf genderte Events warten
				continue;
			} 
		}

		if(DisplayMarquee())
			Interval.Set(SportNGSetup.liveticker.interval*60000); // interval*60*1000 
	}
}

void cLiveTicker::Load(const char* File)
{
#if VDRVERSNUM >= 10503
		cCharSetConv myCharSetConv("ISO-8859-15", cCharSetConv::SystemCharacterTable());
		#define CharConv(s) (char*)(myCharSetConv.Convert(s))
#else
		#define CharConv(s) (char*)s
#endif
	dsyslog("%s: loading %s", PLUGIN_NAME_I18N, File);
	if(FILE *f = fopen(cString::sprintf("%s/%s", DataDir, File), "r"))
	{
		cReadLine ReadLine;
		while(char *Line = ReadLine.Read(f))
		{
			bool found=false;
			char *datas[2];
			SplitString(CharConv(Line), '|', datas, sizeof(datas)/sizeof(datas[0]));
			for(cLiveEvent *Event = First(); Event; Event=Next(Event))
			{
				if(!strcmp(datas[0], Event->Id()))
				{
					found=true;
					Event->SetText(datas[1]);
					break;
				}
			}
			if(!found) Add(new cLiveEvent(datas[0], datas[1]));
		}
	}
}

void cLiveTicker::Update(bool Quiet/*=false*/)
{
	if(cLiveTicker::LiveTicker()==NULL) return;
	cThreadLock ThreadLock(cLiveTicker::LiveTicker());	// wenn Update luft mu Anzeige warten
	cLiveEvent::updated4new=!Quiet;
	// reset all Updated&Touched-Flags
	for(cLiveEvent *Event = First(); Event; Event=Next(Event))
		Event->ClearFlags();
	// update Events
	cLiveTicker::LiveTicker()->Load("SportNG@Live"); // SportNG@Live immer laden
	for(cSetupLine* Setup=SportNGSetup.First(); Setup; Setup = SportNGSetup.Next(Setup))
	{
		if(!strncmp(Setup->Name(), "Liveticker.File.", 16) && atoi(Setup->Value()))
		{
			cLiveTicker::LiveTicker()->Load(&Setup->Name()[16]);
		}
	}
	// delete untouched
	cLiveEvent *NextEvent = First();
	while(NextEvent)
	{
		cLiveEvent *Event = NextEvent; NextEvent=Next(NextEvent);
		if(!Event->Touched()) cLiveTicker::LiveTicker()->Del(Event);
	}
	cLiveTicker::LiveTicker()->Sort();
	cLiveTicker::LiveTicker()->Start();
}

#ifndef OSDHASLEVEL
cLiveTickerOsd::cLiveTickerOsd() : cThread("SportNG-Liveticker-Marquee")
{
}
cLiveTickerOsd::~cLiveTickerOsd()
{
	Cancel(3);
	cLiveTicker::LiveTickerOsd = false;
}
void cLiveTickerOsd::Show()
{
	Start();
}
eOSState cLiveTickerOsd::ProcessKey(eKeys Key)
{
	if(Key == kBack)
	{
		Cancel(3);
		return osBack;
	}
	return osUnknown;
}
#endif /* !OSDHASLEVEL */

#define LIVE_FRAMERATE 50000	/* Framerate in 1/1000 Hz*/ 
#define LIVE_BLINK 500			/* BlinkFrequenz in 1/1000 Hz*/

#ifdef OSDHASLEVEL
bool cLiveTicker::DisplayMarquee()
#else
bool cLiveTicker::DisplayMarquee()
{
	if(!cOsd::IsOpen() && !LiveTickerOsd)
	{
		LiveTickerOsd = true;
		cRemote::CallPlugin(PLUGIN_NAME_I18N);
		return true;
	}
	else
		cCondWait::SleepMs(500);
	return false;
}
void cLiveTickerOsd::Action()
#endif /* OSDHASLEVEL */
{
	cThreadLock ThreadLock(cLiveTicker::LiveTicker());	// wenn Anzeige luft mu Update warten
	const cFont *pFont	= cFont::GetFont(fontOsd);
	int TextHight			= pFont->Height();
	int BinderWidth		= pFont->Width("+ +");
	int OsdWidth			= Setup.OSDWidth;
	int OsdHight			= TextHight+4;
#ifdef OSDHASLEVEL
	cOsd *Osd	= cOsdProvider::NewOsd(Setup.OSDLeft, Setup.OSDTop+Setup.OSDHeight-OsdHight, SportNGSetup.liveticker.osdlevel);
#else
	cOsd *Osd	= cOsdProvider::NewOsd(Setup.OSDLeft, Setup.OSDTop+Setup.OSDHeight-OsdHight);
#endif /* OSDHASLEVEL */
	tArea Area	= { 0, 0, OsdWidth-1, OsdHight-1,  4 };
	Osd->SetAreas(&Area, 1);
	Osd->DrawPixel(0, 0, clrRed); // Reservirt FarbeCode
	int xAll = 0;
	int Blink=0;
	while(Running() && SportNGSetup.liveticker.activ)
	{
#ifdef OSDHASLEVEL
		if(cOsd::IsOpen())			// wenn Osd geffnet 
			cCondWait::SleepMs(500);// dann nur warten (Marquee wird angehalten)
		else
#endif /* OSDHASLEVEL */
		{
			Osd->DrawRectangle(0, 0, OsdWidth-1, OsdHight-1, BAR_COLOR);
			int x = OsdWidth-1-xAll;
			bool drawed=false;
			for(int Loop = 0; Loop < SportNGSetup.liveticker.repeats; Loop++)
			{
				cLiveEvent *Event;
				for(Event = cLiveTicker::First(); Event; Event=cLiveTicker::Next(Event))
				{
					if(Event->Updated() || SportNGSetup.liveticker.display)
					{
						if(x>-BinderWidth && x<OsdWidth)
						{
							Osd->DrawText(x, 2, "++ ", TEXT_COLOR, clrTransparent, pFont /*, int Width = 0, int Height = 0, int Alignment = taDefault*/);
							drawed=true;
						}
						else if(x>=OsdWidth)
							break;
						x += BinderWidth;
						int TextWidth = pFont->Width(Event->Text());
						if(x>-TextWidth && x<OsdWidth)
						{
							int Fade = (abs(0x100-Blink)<<2)-128; // Lnger Schwarz als Transp.
							tColor BlinkColor = FadeColor(BAR_COLOR, EVENT_COLOR, Fade);
							Osd->DrawText(x, 2, Event->Text(), (Event->Updated() && SportNGSetup.liveticker.display ? BlinkColor : TEXT_COLOR), clrTransparent, pFont /*, int Width = 0, int Height = 0, int Alignment = taDefault*/);
							drawed=true;
						}
						else if(x>=OsdWidth)
							break;
						x += TextWidth;
						if(x>-BinderWidth && x<OsdWidth)
						{
							Osd->DrawText(x, 2, " ++", TEXT_COLOR, clrTransparent, pFont /*, int Width = 0, int Height = 0, int Alignment = taDefault*/);
							drawed=true;
						}
						else if(x>=OsdWidth)
							break;
						x += BinderWidth;
					}
				}
				if(Event)	break;	 // Schleife abgebrochen
				x+=200; //200 Pixel Abstand
			}
			if(drawed)
			{
				Osd->Flush();
				cCondWait::SleepMs(1000000/LIVE_FRAMERATE);
				xAll += SportNGSetup.liveticker.speed+1;
			}
			else
			{
				break;
			}
		}
		Blink = (Blink + max(((0x200 * LIVE_BLINK)/LIVE_FRAMERATE),1))%0x0200;
	}
	for(cLiveEvent *Event = cLiveTicker::First(); Event; Event=cLiveTicker::Next(Event))
		Event->ClearFlags();
	DELETENULL(Osd);
#ifdef OSDHASLEVEL
	return true;
#else
	cRemote::Put(kBack, true);
#endif /* OSDHASLEVEL */
}

