/*
 * Sportng.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <getopt.h>

#include <vdr/plugin.h>

#include "setup.h"
#include "sportngosd.h"
#include "liveticker.h"
#include "sportngtools.h"
// #include "i18n.h"

// ------------- cMenu SportNG Setup ------------------------------ //
class cMenuSportNGSetup;
class cMenuEditSportNGAdjustItem : public cMenuEditIntItem
{
private:
	cMenuSportNGSetup* inMenuSportNGSetup;
public:
	cMenuEditSportNGAdjustItem(cMenuSportNGSetup* InMenuSportNGSetup, const char *Name, int *Value, int Min, int Max)
																									:
																									cMenuEditIntItem(Name, Value, Min, Max) ,inMenuSportNGSetup(InMenuSportNGSetup)
	{
	}
	eOSState ProcessKey(eKeys Key);
};
class cMenuSportNGSetup : public cMenuSetupPage {
private:
	cSportNGSetup data;
	int	tab_pos;
	const char *update_strings[3];

	int text_area_width;
	const cFont *font;
	int space_width;
	static const char *arrow;
	int arrow_width;
	char *arrow_line;
	int arrow_line_len;
	cOsdItem *adjust1;
protected:
	virtual void Store(void);
public:
	cMenuSportNGSetup(void)
	{
		data = SportNGSetup;

		Display();
		text_area_width	= DisplayMenu()->GetTextAreaWidth();
		font					= DisplayMenu()->GetTextAreaFont(false);
		if (!font) font	= cFont::GetFont(fontOsd);
		
		space_width			= font->Width(' ');
		arrow_width			= font->Width(arrow);
		arrow_line_len		= text_area_width/space_width;
		arrow_line			= MALLOC(char, arrow_line_len + strlen(arrow)+1 /*+1=>\0*/);
									memset(arrow_line, ' ', arrow_line_len);
									strcpy(&arrow_line[arrow_line_len], arrow);
		
		Add(new cMenuEditSportNGAdjustItem(this, tr("Adjust 1"), &data.s_adjust1,3-((text_area_width-arrow_width)/space_width),3));
		Add(adjust1 = new cOsdItem(&arrow_line[arrow_line_len-((text_area_width-arrow_width)/space_width)+3], osUnknown, false));

		Add(new cMenuEditSportNGAdjustItem(this, tr("Adjust 2"), &data.s_adjust2,-20,7));
		Add(new cOsdItem(cString::sprintf("\t%s", arrow), osUnknown, false));
		Add(new cOsdItem("---- Liveticker -------------------------------------------------------------", osUnknown, false));
		Add(new cMenuEditBoolItem(tr("Liveticker"), &data.liveticker.activ));
#ifdef OSDHASLEVEL
		Add(new cMenuEditIntItem(tr("  OSD Level"), &data.liveticker.osdlevel, 1, 99));
#endif
		Add(new cMenuEditIntItem(tr("  Interval (Min.)"), &data.liveticker.interval, 0, 20, tr("on changed Event")));
		Add(new cMenuEditBoolItem(tr("  Display Live-Events"), &data.liveticker.display, tr("Changed"), tr("All")));
		static const char * const Speed[] = {"1x", "2x", "3x", "4x"};
		Add(new cMenuEditStraItem(tr("  Marquee-Speed"), &data.liveticker.speed, 4, Speed));
		Add(new cMenuEditIntItem(tr("  Repeats"), &data.liveticker.repeats, 1, 10));
		AdjustChanged();
	}
	~cMenuSportNGSetup(void)
	{
		free(arrow_line);
	}
	void AdjustChanged(void)
	{
		adjust1->SetText(&arrow_line[arrow_line_len-((text_area_width-arrow_width)/space_width)+3-data.s_adjust1]);
		cMenuSetupPage::SetCols((text_area_width-((3-data.s_adjust1)*space_width)-arrow_width)/(max(14-data.s_adjust2,5)));
		Display();
	}
};
const char * cMenuSportNGSetup::arrow = "=====>|";
eOSState cMenuEditSportNGAdjustItem::ProcessKey(eKeys Key)
{
	eOSState state = cMenuEditIntItem::ProcessKey(Key);
	inMenuSportNGSetup->AdjustChanged();
//	inMenuSportNGSetup->Display();
	return state;
}

// -------------- cPluginSportNG ----------------------------------- //

#ifdef SPORTNG_DEVEL
static const char *VERSION				= "0.0.3-devel";
#else
static const char *VERSION				= "0.0.3";
#endif
static const char *DESCRIPTION		= trNOOP("Zeigt Sportergebnisse auf dem OSD");
static const char *MAINMENUENTRY		= trNOOP("SportNG");
const char *SportNG_VERSION=VERSION;
class cPluginSportNG : public cPlugin, public cThread
{
private:
	bool refresh;
protected:
	void Action(void);
public:
	cPluginSportNG(void);
	virtual ~cPluginSportNG();
	virtual const char *Version(void) { return VERSION; }
	virtual const char *Description(void) { return tr(DESCRIPTION); }
	virtual const char *CommandLineHelp(void);
	virtual bool ProcessArgs(int argc, char *argv[]);
	virtual bool Initialize(void);
	virtual bool Start(void);
	virtual void Stop(void);
	virtual void Housekeeping(void);
	virtual cString Active(void);
	virtual const char *MainMenuEntry(void) { return tr(MAINMENUENTRY); }
	virtual cOsdObject *MainMenuAction(void);
	virtual cMenuSetupPage *SetupMenu(void);
	virtual bool SetupParse(const char *Name, const char *Value);
	virtual bool Service(const char *Id, void *Data = NULL);
	virtual const char **SVDRPHelpPages(void);
	virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
};

cPluginSportNG::cPluginSportNG(void) : cThread("SportNG-Update")
{
	refresh = false;
	SportNGSetup.SetPlugin(this);
}

cPluginSportNG::~cPluginSportNG()
{

}

const char *cPluginSportNG::CommandLineHelp(void)
{
	static char *help_str = NULL;

	free(help_str);
	asprintf(&help_str, 	"	-D DIR		--data=DIR		DIR to datafiles\n"
				 				"	-S DIR		--script=DIR	DIR to downloadscript\n");
	return help_str;
}

bool cPluginSportNG::ProcessArgs(int argc, char *argv[])
{
	static struct option long_options[] = {
			 { "data",					required_argument, NULL, 'D' },
			 { "script",				required_argument, NULL, 'S' },
			 { NULL }
		 };

	int c, option_index = 0;
	while ((c = getopt_long(argc, argv, "D:S:", long_options, &option_index)) != -1)
	{
		switch (c)
		{
			case 'D': DataDir		 = optarg; break;
			case 'S': ScriptDir	 = optarg; break;
	 		default: return false;
		}
	}
	return true;
}

bool cPluginSportNG::Initialize(void) {

#if VDRVERSNUM < 10507
// 	RegisterI18n(Phrases);
#endif
	if (DataDir==NULL)
	{
//		DataDir = strdup(ConfigDirectory(PLUGIN_NAME_I18N));
		asprintf(&DataDir, "%s/data", ConfigDirectory(PLUGIN_NAME_I18N));
		dsyslog(	"%s: Parameter '-D' not set.\n"
					" Set default value : %s\n", PLUGIN_NAME_I18N, DataDir);
	}
	if (DataDir!=NULL)
	{
		char* buffer;
		asprintf(&buffer, "mkdir %s", DataDir);
	 	dsyslog("%s: system('%s');\n", PLUGIN_NAME_I18N, buffer);
		system(buffer);
		free(buffer);
	}

	if (ScriptDir==NULL)
	{
		ScriptDir = strdup(ConfigDirectory(PLUGIN_NAME_I18N));
		dsyslog("%s: Parameter '-S' not set.\n"
						" Set default value : %s\n", PLUGIN_NAME_I18N, ScriptDir);
		char* buffer;
		asprintf(&buffer, "%s/sportng.sh", ScriptDir);
//		cString xbuffer=cString::sprintf("%s/sportng.sh", ScriptDir);

		dsyslog("%s: Create : %s\n", PLUGIN_NAME_I18N, buffer);
		if (access(buffer, F_OK) != 0 || 1)
		{
			FILE *f = fopen(buffer, "w");
			if (f)
			{
//				fwrite(Script, sizeof(char), strlen(Script), f);
extern const char	SportNGScript[];
extern const int SportNGScript_length;
				fwrite(SportNGScript, sizeof(char), SportNGScript_length, f);
				fclose(f);
			}
			chmod(buffer, S_IRWXU|S_IRWXG|S_IRWXO);
		}
		free(buffer);
	}
	return true;
}

bool cPluginSportNG::Start(void)
{
	new cLiveTicker;
	return cThread::Start();
}

void cPluginSportNG::Stop(void)
{
	if(cLiveTicker::LiveTicker())
		delete cLiveTicker::LiveTicker();
	cThread::Cancel(3);
}

void cPluginSportNG::Housekeeping(void)
{

}
cString cPluginSportNG::Active(void)
{
	if(refresh)
		return tr("SportNG Data-Download left");
	return NULL;
}

cOsdObject *cPluginSportNG::MainMenuAction(void)
{
#ifndef OSDHASLEVEL
	if(!cLiveTicker::LiveTickerOsd)
#endif /* !OSDHASLEVEL */
		return new cSportNG("SportNG");
#ifndef OSDHASLEVEL
	else
		return new cLiveTickerOsd;
#endif /* !OSDHASLEVEL */

}


bool cPluginSportNG::Service(const char *Id, void *Data)
{
	return false;
}

const char **cPluginSportNG::SVDRPHelpPages(void)
{
	return NULL;
}

cString cPluginSportNG::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
{
	return NULL;
}

cMenuSetupPage *cPluginSportNG::SetupMenu(void)
{
	return new cMenuSportNGSetup;
}

bool cPluginSportNG::SetupParse(const char *Name, const char *Value)
{
	if			(!strcasecmp(Name, "Adjust1"))					SportNGSetup.s_adjust1				= atoi(Value);
	else if	(!strcasecmp(Name, "Adjust2"))					SportNGSetup.s_adjust2				= atoi(Value);
	else if	(!strcasecmp(Name, "Liveticker.Activ"))		SportNGSetup.liveticker.activ		= atoi(Value);
	else if	(!strcasecmp(Name, "Liveticker.Osdlevel"))	SportNGSetup.liveticker.osdlevel	= atoi(Value);
	else if	(!strcasecmp(Name, "Liveticker.Interval"))	SportNGSetup.liveticker.interval	= atoi(Value);
	else if	(!strcasecmp(Name, "Liveticker.Display"))		SportNGSetup.liveticker.display	= atoi(Value);
	else if	(!strcasecmp(Name, "Liveticker.Speed"))		SportNGSetup.liveticker.speed		= atoi(Value);
	else if	(!strcasecmp(Name, "Liveticker.Repeats"))		SportNGSetup.liveticker.repeats	= atoi(Value);
	else
	{
		SportNGSetup.SetPlugin(NULL); // no plugin->SetupStore in SportNGSetup
		SportNGSetup.Store(Name, Value);
		isyslog(	"%s: variable Setup (%s=%s)", PLUGIN_NAME_I18N, Name, Value);
		SportNGSetup.SetPlugin(this); // restore SetupStore.plugin
		return true;
	}
	return true;
}
void cMenuSportNGSetup::Store(void)
{
	SportNGSetup = data;
	SetupStore("Adjust1",										SportNGSetup.s_adjust1);
	SetupStore("Adjust2",										SportNGSetup.s_adjust2);
	SetupStore("Liveticker.Activ",							SportNGSetup.liveticker.activ);
	SetupStore("Liveticker.Osdlevel",						SportNGSetup.liveticker.osdlevel);
	SetupStore("Liveticker.Interval",						SportNGSetup.liveticker.interval);
	SetupStore("Liveticker.Display",							SportNGSetup.liveticker.display);
	SetupStore("Liveticker.Speed",							SportNGSetup.liveticker.speed);
	SetupStore("Liveticker.Repeats",							SportNGSetup.liveticker.repeats);
}

void cPluginSportNG::Action(void)
{
	cLiveTicker::Update(/*Quiet=*/true); //1.Update mit Quiet damit alter Bestand nicht angezeigt wird
	cTimeMs Reload;
	while(Running())
	{
		if(time(NULL)>GetNextUpdate() && Reload.TimedOut())
//		if(Reload.TimedOut())
		{
			refresh = true;

			dsyslog("%s: Refresh started", PLUGIN_NAME_I18N);
			SystemExec(cString::sprintf("'%s/%s' download '%s' 'sportng=%s'", ScriptDir, "sportng.sh", DataDir, SportNG_VERSION));
			GetNextUpdate(true); // SportNG@NextUpdate neu einlesen
			dsyslog("%s: Refresh completed", PLUGIN_NAME_I18N);

			if(SportNGSetup.liveticker.activ)	cLiveTicker::Update();
				
			refresh = false;
			Reload.Set(180000); // frhestens in 3 Minuten 
//			Reload.Set(20000); // frhestens in 3 Minuten 
		}
		cCondWait::SleepMs(500); // schlaf mein Kindchen schlaf ... ;-)
	}
}

VDRPLUGINCREATOR(cPluginSportNG); // Don't touch this!
