
#include <string>
#include <fstream>
#include <vdr/menu.h>

#include <vdr/osd.h>
#include <vdr/menuitems.h>
#include <vdr/remote.h>

#include "sportngosd.h"
#include "liveticker.h"
#include "sportngtools.h"
#include "setup.h"
// #include "i18n.h"
#if 0
static const char* translate[]= {trNOOP("Test tr-String"),trNOOP("Test tr-String2")}
#endif
extern const char *SportNG_VERSION;

/* ########################################################
   ------------------- Men Refresh------------------------
   ######################################################## */

class cSportNGRefresh : public cOsdMenu
{
public:
	cSportNGRefresh(int Mode, const void* Var=NULL);
	enum {REFRESH, UPDATE};
private:
	eOSState Reload();
	eOSState Update(const char* File);
	void Completed(const char *Message);
	bool ExecCommand(const char *Message, const char *Command, bool NiceLine=true);
	eOSState ProcessKey(eKeys Key);
};

/* ########################################################
   ------------------- HelpKeys ---------------------------
   ######################################################## */

cSportNGHelpKeys::cSportNGHelpKeys()
{
	for(int i=0;i<4;i++)	helpKeys[i]=helpCommands[i]=NULL;
}

//##################################################################

cSportNGHelpKeys::~cSportNGHelpKeys()
{
	ClearHelpKeys();
}

//##################################################################

void cSportNGHelpKeys::SetHelpKeyAndCommand(int Idx, const char *HelpKey, const char* Command)
{
	if(0<=Idx && Idx<=3)
	{
		free(helpKeys[Idx]);
		helpKeys[Idx] = HelpKey ? strdup(HelpKey) : NULL;
		free(helpCommands[Idx]);
		helpCommands[Idx] = Command ? strdup(Command) : NULL;
	}
}

//##################################################################

void cSportNGHelpKeys::SetHelpKeyAndCommand(int Idx, char *KeyAndCommand)
{
	if(0<=Idx && Idx<=3 && KeyAndCommand)
	{
		char *c=0;
		if(KeyAndCommand[0]=='*' && KeyAndCommand[1]==0)
			c = KeyAndCommand;
		else if(KeyAndCommand && (c=strchr(KeyAndCommand, '=')))
			*c++=0;
		if(c)
		{
			SetHelpKeyAndCommand(Idx, KeyAndCommand, c);
		}
	}
}

//##################################################################

void	cSportNGHelpKeys::ClearHelpKeys()
{
	for(int i=0; i<4;i++)	// HelpKeys Clear
	{
		free(helpKeys[i]); helpKeys[i]=NULL;
		free(helpCommands[i]); helpCommands[i]=NULL;
	}
}

//##################################################################

const char *cSportNGHelpKeys::GetHelpCommand(int Idx)	// 0=red, 1=green, 2=yellow, 3=blue 
{
	if(0<=Idx && Idx<=3)
		if(helpKeys[Idx])
			return helpCommands[Idx];
	return NULL;
}

//##################################################################

char **cSportNGHelpKeys::GetHelpKeys(char *HelpKeys[], bool CopyAll=true)
{
	for(int i=0;i<4;i++)
	{
		if(helpKeys[i] || CopyAll)
		{
			if(helpKeys[i] && helpKeys[i][0]=='*')
				HelpKeys[i] = NULL;
			else
				HelpKeys[i] = helpKeys[i];
		}
	}
	return HelpKeys;	
}

/* ########################################################
   ------------------- cSportNGBase -----------------------
   ######################################################## */

cSportNGBase::cSportNGBase()
{
	pFileName=0;
	FileTime=0;
};

//##################################################################

cSportNGBase::~cSportNGBase()
{
	free(pFileName);
};

//##################################################################

bool cSportNGBase::LoadContent(const char *File)
{
	bool reLoad = true;
	if(pFileName != File)
	{
		reLoad = false;
		free(pFileName);
		if(File)
			pFileName = strdup(File);
		else
			pFileName = 0;
	}
	bool ret=true;
	if(pFileName)
	{
		ret=false;
		unsigned int FileSize=0;
		time_t newFileTime=0;
		cString FullFileName = cString::sprintf("%s/%s", DataDir, pFileName);
		struct stat buf;
		if(stat(FullFileName,&buf)==0)
		{
			newFileTime	= buf.st_mtime;
			FileSize		= buf.st_size;
		}
		if(reLoad && FileTime == newFileTime)	// wenn die selbe Datei geladen werden Soll (reLoad==true)
			return true;								// aber das Datum gleich ist ---> nichts tun 
		FileTime = newFileTime;

		FILE *f = fopen(FullFileName, "r");
		if(f)
		{
			isyslog("%s: loading '%s'", PLUGIN_NAME_I18N, pFileName);
			if(char* buffer = MALLOC(char, FileSize+1))
			{
				if(fread(buffer, 1, FileSize, f)==FileSize)
				{
					buffer[FileSize] = 0;
					SetContent(buffer, /*i18n=*/true , /*fromLoad=*/true );
					ret=true;
				}
				free(buffer);
			}
			fclose(f);
		}
		else
			esyslog("%s: Can't open '%s'\n",PLUGIN_NAME_I18N, pFileName);
	}
	return ret;
}

//##################################################################

extern const char	SportNGInfo[];
eOSState cSportNGBase::ParseCommand(const char* Command)
{
	if(Command)
	{
		switch(*Command)
		{
			case 'b':	// Back
				return osBack;
				break;
			case 'i':	// Info
				return AddSubMenu (new cSportNGMenuText(SportNGInfo, true));
			case 'l':	// Load new Content
	isyslog("%s: load-command %s", PLUGIN_NAME_I18N, &Command[1]);
				LoadContent(&Command[1]);	
				break;
			case 'r':	// Download new Data
				return AddSubMenu(new cSportNGRefresh(cSportNGRefresh::REFRESH));
				break;
			case 's':	// open SubMen
				return AddSubMenu(new cSportNG(&Command[1]));
			case 't':	// open TextSubMen
				return AddSubMenu(new cSportNGMenuText(&Command[1]));
			case 'u':	// Update
				return AddSubMenu(new cSportNGRefresh(cSportNGRefresh::UPDATE, &Command[1]));
			case 'x':	// Exit
				return osEnd;
				break;
			default:
				;	
		}
	}
	return osContinue;
}

/* ########################################################
   ------------------- SportNG Item -----------------------
   ######################################################## */

class cSportNGItem : public cSportNGHelpKeys, public cOsdItem 
{
private:
	char *command;
public:
	cSportNGItem(char *Datas[]) : cOsdItem(Datas[1], osUnknown, true)
	{
		command=Datas[2] ? strdup(Datas[2]) : NULL;
		for(int i=0;i<4;i++)
			SetHelpKeyAndCommand(i, Datas[i+3]);
	}
	cSportNGItem(const char *Text) : cOsdItem(Text, osUnknown, false)
	{
		command = NULL;
	}
	~cSportNGItem()
	{
		free(command);
	}
	const char* GetCommand(void)
	{
		return command;
	}
};

/* ########################################################
   -------------- SportNG SetupItem  ----------------------
   ######################################################## */
#define SETUPMAXOPTIONS 32   
class cSetupItem : public cSportNGHelpKeys, public cOsdItem 
{
private:
	char *stext;
	char *command;
	char *setup;
	char *option[SETUPMAXOPTIONS];
	int	max_option;
	int	act_option;
public:
	cSetupItem(char *Datas[]) : cOsdItem("", osUnknown, true)
	{
		int i;
		stext = strdup(Datas[1]);
		command=Datas[2] ? strdup(Datas[2]) : NULL;
		for(i=0;i<4;i++)
			SetHelpKeyAndCommand(i, Datas[i+3]);
		setup = strdup(Datas[7]);
		option[0] = NULL;
		max_option = SplitString(strdup(Datas[8]), '/', option, sizeof(option)/sizeof(option[0]))-1;
		act_option = 0;
		if(cSetupLine *SetupLine = SportNGSetup.Get(Datas[7]))
			act_option = min(max_option, atoi(SetupLine->Value()));
		Set();
	}
	~cSetupItem()
	{
		free(stext);
		free(command);
		free(setup);
		free(option[0]);
	}
	const char* GetCommand(void)
	{
		return command;
	}
	virtual eOSState ProcessKey(eKeys Key)
	{
		eOSState state = cOsdItem::ProcessKey(Key);
		
		if (state == osUnknown)
		{
			state = osContinue;
			switch (Key)
			{
				case kLeft:
					if(--act_option < 0)
						act_option = 0;
					break;
				case kRight:
					if(++act_option > max_option)
						act_option = max_option;
					break;
				default:
					return osUnknown;
					break;
			}
			Set();
		}
		return state;
	}
private:
	void Set(void)
	{
		SetText(cString::sprintf("%s%s%s%s", tr(stext), act_option==0?"":"<", tr(option[act_option]), act_option==max_option?"":">"));
		SportNGSetup.Store(setup, act_option);
	}
};


/* ########################################################
   -------------- H A U P T M E N  -----------------------
   ######################################################## */

const char *EM[] = {
			trNOOP("Error while loading"),
			trNOOP("Error while loading"),
			trNOOP("File:%s"),
			trNOOP("Check your systemlog for errormessages."),
			trNOOP("Try other paths with commandline flags"),
			trNOOP("If nothing helps, get an internetconnection.")
};


//class cSportNGRefresh;
cSportNG::cSportNG(const char *File)
:cOsdMenu("")
{
	if(!File) return;
	lastItem = -1;
	Display(); // !!! WICHTIG damit DisplayMenu()->GetTextAreaWidth() korrekten Wert liefert
	isyslog("%s: Load Menu from '%s'\n",PLUGIN_NAME_I18N, File);
	if(LoadContent(File))
	{
		isyslog("%s: Loaded Menu from '%s'\n",PLUGIN_NAME_I18N, File);
	}
	else
	{
		esyslog("%s: Can't load Menu from '%s'\n",PLUGIN_NAME_I18N, File);
		SetContent(cString::sprintf("t|%s\nl|%s\nm|%s\nl|%s\nl|%s\nl|%s",
								tr(EM[0]), tr(EM[1]), (const char*)cString::sprintf(tr(EM[2]), File), tr(EM[3]), tr(EM[4]), tr(EM[5])
						), /*i18n=*/false , /*fromLoad=*/true );
		static bool FirstStart = true;
		if(!strcmp(File, "SportNG") && FirstStart)
		{
			FirstStart = false;
			AddSubMenu(new cSportNGRefresh(cSportNGRefresh::REFRESH));
		}
	}
	Display();
	DisplayHelpKeys();
	
}
//##################################################################
/*
cSportNG::~cSportNG()
{
}
*/

//##################################################################
int cSportNG::GetTextAreaWidth()
{
	const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
	if (!pFont) pFont = cFont::GetFont(fontOsd);
	return DisplayMenu()->GetTextAreaWidth() - (3-SportNGSetup.s_adjust1)*pFont->Width(' ');
}
//###########################################################
/*
#define AverageCharacterWidth	(max(14-SportNGSetup.s_adjust2,5))
int cSportNG::GetAverageCharacterWidth(void)
{
	const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
	if (!pFont) pFont = cFont::GetFont(fontOsd);
	return ::GetAverageCharacterWidth(pFont, SportNGSetup.s_adjust2);
}
*/
//###########################################################

void cSportNG::CalcColWidth(const char *Text, int ColWidth[6], int LastColWidth[6])
{
	if(char *text = strdup(Text))
	{
		const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
		if (!pFont) pFont = cFont::GetFont(fontOsd);
		char *s = text;
		for(int i=0; i<6; i++)
		{
			char *e = strchrnul(s, '\t');
			if(*e)
			{
				*e++=0;
				int width = pFont->Width(s);
				if(ColWidth[i]<width)
					ColWidth[i]=width;
				 s=e;
			}
			else
			{
				if(*s)
				{
					int width = pFont->Width(s);
					if(LastColWidth[i]<width)
						LastColWidth[i]=width;
				}
				break;
			}
		}
		free(text);
	}
}
char* cSportNG::GetSetupText(const char *Text, const char* Options)
{
	const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
	if (!pFont) pFont = cFont::GetFont(fontOsd);
	
	int MaxOptionWidth= 0;

	char *option[SETUPMAXOPTIONS];
	int	max_option = SplitString(strdup(Options), '/', option, sizeof(option)/sizeof(option[0]))-1;
	for(int i=0; i<=max_option; i++)
	{
		int opt_width = pFont->Width(option[i]);
		if(MaxOptionWidth < opt_width)
			MaxOptionWidth = opt_width;
	}
	free(option[0]);


	int filllen = cSportNG::GetTextAreaWidth()-pFont->Width(Text)-MaxOptionWidth-pFont->Width("< >");
	if(filllen < 0) filllen=0;
	else
		filllen /= pFont->Width(' ');
	
	if(filllen)
	{
		int textlen	= strlen(Text);
		int len		= textlen+filllen;
		char *ret	= MALLOC(char, len+1);	strcpy(ret, Text); memset(ret+textlen, ' ', filllen);
		ret[len]		= '\0';
		return ret;
	}
	return strdup(Text);
}

//##################################################################

void cSportNG::SetContent(const char *Text, bool i18n/*=true*/, bool fromLoad/*=false*/)
{
	if(!fromLoad)
		LoadContent(NULL);

	Clear();
	Skins.Flush();
	ClearHelpKeys();
	SetHelpKeyAndCommand(2, tr("Refresh"),"r");	// HelpKeys Std
	SetHelpKeyAndCommand(3, tr("Info"), "i");
	if(Text)
	{
#if VDRVERSNUM >= 10503
		cCharSetConv myCharSetConv("ISO-8859-15", cCharSetConv::SystemCharacterTable());
		#define CharConv(s) (char*)(i18n?myCharSetConv.Convert(s):s)
#else
		#define CharConv(s) (char*)s
#endif
		int ColWidth[6]		= {0, 0, 0, 0, 0, 0};
		char ColFlags[6]		= {'*','*','*','*','*','*'};
		int LastColWidth[6]	= {0, 0, 0, 0, 0, 0};
		char *s, *n, *text = n = strdup(Text); // weiter mit Text-Copy
		int line = 0;
		SetHasHotkeys();

		while(n)
		{
			s = n;
			n = strchr(s, '\n');
			if(n) *n++ = 0;
			
			line++;
			if (!isempty(s))
			{
				if(s[0] && s[1]=='|')
				{
					s = CharConv(s);
					// m|MenText|OK-Command|redKey=redKeyCommand|greenKey=greenKeyCommand|yellowKey=yellowKeyCommand|blueKey=blueKeyCommand
					// s|MenText|OK-Command|redKey=redKeyCommand|greenKey=greenKeyCommand|yellowKey=yellowKeyCommand|blueKey=blueKeyCommand|ConfigEntry|Values
					char *datas[9];

					int Parts = SplitString(s, '|', datas, sizeof(datas)/sizeof(datas[0]));
					switch(datas[0][0])
					{
					case 'c':	// Col-Flags '+'=darf vergrert werden '-'=darf verkleinert werden '*'=darf vergrert und verkleinert werden
						{
							int i;
							for(i=0; i<6; i++)
								ColFlags[i] = datas[i+1]&&datas[i+1][0] ? datas[i+1][0] : '*';
						}
						break;
					case 'f':	// Hilfs-Zeile fr Formatierung
						CalcColWidth(datas[1], ColWidth, LastColWidth);
						break;
					case 'h':	// Help-Keys setzen
						{
							for(int i=0;i<4;i++)	SetHelpKeyAndCommand(i, datas[i+1]);
						}
						break;
					case 'l':	// normale Zeile
						CalcColWidth(datas[1], ColWidth, LastColWidth);
						Add(new cSportNGItem(datas[1]));
						break;
					case 'm':	// Men
						CalcColWidth(datas[1], ColWidth, LastColWidth);
						Add(new cSportNGItem(datas));
						break;
					case 's':	// Setup
						if(Parts == 9)
						{
							datas[1] = GetSetupText(datas[1], datas[8]);
							Add(new cSetupItem(datas));
							free(datas[1]);
						}
						break;
					case 't':	// Titel setzen
						SetTitle(datas[1]);
						break;
					default:
						;
					}
				}
			}
		}
		free(text);
		const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
		if (!pFont) pFont = cFont::GetFont(fontOsd);
		int AreaWidth = GetTextAreaWidth()/AverageCharacterWidth;	// Gesamtbreite in AverageChars (abgerundet)
		int ColWidthSum=0;

		int i;
		for(i=0; i<6; i++)
			if(ColWidth[i] == 0) ColWidth[i] = LastColWidth[i];
		for(i=0; i<6 && ColWidth[i]; i++)
		{
			ColWidth[i] = (ColWidth[i]+AverageCharacterWidth-1)/AverageCharacterWidth;	// Breite in AverageChars (aufgerundet)
			ColWidthSum += ColWidth[i];
		}
		if(ColWidthSum != AreaWidth) // Spalten mssen vergrert/verkleinert werden
		{
			char cmp;
			if(ColWidthSum < AreaWidth)
				cmp='+'; // Spalten mssen vergrert werden
			else
				cmp='-'; // Spalten mssen verkleinert werden
						
			for(int i=0; i<6 && ColWidth[i]; i++)
			{
				if(ColFlags[i] != '+' && ColFlags[i] != '*')
				{
					AreaWidth -= ColWidth[i];			// Cols, die nicht vergrert/verkleinert werden drfen aus Summen entfernen
					ColWidthSum -= ColWidth[i];
				}
			}
			int Rest = 0;
			for(int i=0; i<6 && ColWidth[i]; i++)
			{
				if(ColFlags[i] == '+' || ColFlags[i] == '*')
				{
					ColWidth[i] = ((ColWidth[i]*AreaWidth<<8)/ColWidthSum)+128+Rest;			// Cols, die vergrert/verkleinert werden drfen anpassen
					Rest = (ColWidth[i]&0xff)-128;														// RundungsDifferenz mitnehmen
					ColWidth[i] >>= 8;
				}
			}
		}
		SetCols(ColWidth[0], ColWidth[1], ColWidth[2], ColWidth[3], ColWidth[4]);

#if 0
Add(new cSportNGItem(cString::sprintf("ColWidth=%i,%i,%i,%i,%i,%i ", ColWidth[0], ColWidth[1], ColWidth[2], ColWidth[3], ColWidth[4], ColWidth[5])));

{
char line[17]; line[16]=0;
for(int l=32; l<256; l+=16)
{
for(int s=0; s<16;s++)
{
	line[s] = l+s;
}
	Add(new cSportNGItem(CharConv(cString::sprintf("Line %i = %s", l, line))));
}}

char* dummy[]={"","dummyTXT",NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
Add(new cSportNGItem(dummy));
#endif 
	}
	Display();
	DisplayHelpKeys(true);
}

//########################################################

void cSportNG::DisplayHelpKeys(bool Force)
{
	if(lastItem != Current() || Force)
	{
		lastItem = Current();
		char *currenthelpKeys[4];
		GetHelpKeys(currenthelpKeys);
		cSportNGItem *item = (cSportNGItem*)Get(Current());
		if(item)	item->GetHelpKeys(currenthelpKeys, false);
		SetHelp(currenthelpKeys[0],currenthelpKeys[1],currenthelpKeys[2],currenthelpKeys[3]);
	}
}

//#############################################################

eOSState cSportNG::ProcessKey(eKeys Key)
{
	bool hadSubmenu = HasSubMenu();

	eOSState state = cOsdMenu::ProcessKey(Key);

	if(hadSubmenu && !HasSubMenu())
	{
		if(GetFileName()) LoadContent(GetFileName());
		return osContinue;
	}

	if(state==osContinue)
		DisplayHelpKeys();// HelpKeys am Kontext des aktuellen Items anpassen
	else if(state==osUnknown)
	{
		state = osContinue;
		const char *Command=NULL;
		cSportNGItem *item = (cSportNGItem*)Get(Current());
		switch (Key)
		{
		case kRed:
			if(item)	Command = item->GetHelpCommand(0);
			if(!Command)	Command = GetHelpCommand(0);
			if(Command)	return ParseCommand(Command);
			break;
		case kGreen:
			if(item)	Command = item->GetHelpCommand(1);
			if(!Command)	Command = GetHelpCommand(1);
			if(Command)	return ParseCommand(Command);
			break;
		case kYellow:
			if(item)	Command = item->GetHelpCommand(2);
			if(!Command)	Command = GetHelpCommand(2);
			if(Command)	return ParseCommand(Command);
			break;
		case kBlue:
			if(item)	Command = item->GetHelpCommand(3);
			if(!Command)	Command = GetHelpCommand(3);
			if(Command)	return ParseCommand(Command);
			break;
		case kOk:
			if(item)	Command = item->GetCommand();
			if(Command)	return ParseCommand(Command);
			break;
		default:
			;
		}
	}
	return state;
}
//###########################################################


/* ########################################################
   ------------------- cSportNGMenuText -------------------
   ######################################################## */

//##################################################################

cSportNGMenuText::cSportNGMenuText(const char *File, bool isText) : cMenuText("", NULL)
{
	if(isText)
		SetContent(File);
	else if(!LoadContent(File))
	{
		esyslog("%s: Can't load Menu from '%s'\n",PLUGIN_NAME_I18N, File);
		SetContent(cString::sprintf("%s|||||%s\n%s\n%s\n%s\n%s",
								tr(EM[0]), tr(EM[1]), (const char*)cString::sprintf(tr(EM[2]), File), tr(EM[3]), tr(EM[4]), tr(EM[5])
						), /*i18n=*/false , /*fromLoad=*/true );
		Skins.Message(mtError, tr("Error while loading"), 2);
	}
}

//##################################################################

void cSportNGMenuText::SetContent(const char *Text, bool i18n/*=true*/, bool fromLoad/*=false*/)
{
	ClearHelpKeys();
	SetHelpKeyAndCommand(2, tr("Refresh"),"r");	// HelpKeys Std
	SetHelpKeyAndCommand(3, tr("Info"), "i");
#if VDRVERSNUM >= 10503
	cCharSetConv myCharSetConv("ISO-8859-15", cCharSetConv::SystemCharacterTable());
	#define CharConv(s) (char*)(i18n?myCharSetConv.Convert(s):s)
#else
	#define CharConv(s) (char*)s
#endif
	if(Text)
	{
		char *text = strdup(Text); // weiter mit Text-Copy
		char *datas[6];
		SplitString(CharConv(text), '|', datas, sizeof(datas)/sizeof(datas[0]));

		SetTitle(datas[0]);
		for(int i=0;i<4;i++)	SetHelpKeyAndCommand(i, datas[i+1]);
		SetText(datas[5]);
		free(text);
	}
	DisplayHelpKeys();
}

//##################################################################

void cSportNGMenuText::DisplayHelpKeys()
{
	char *currenthelpKeys[4];
	GetHelpKeys(currenthelpKeys);
	SetHelp(currenthelpKeys[0],currenthelpKeys[1],currenthelpKeys[2],currenthelpKeys[3]);
}

//##################################################################

eOSState cSportNGMenuText::ProcessKey(eKeys Key)
{
	eOSState state=osUnknown;
	bool hadSubmenu = HasSubMenu();

	switch (Key)
	{
	case kRed:
	case kGreen:
	case kYellow:
	case kBlue:
		{
			state = cOsdMenu::ProcessKey(Key);
		
			if(hadSubmenu && !HasSubMenu())
				return osContinue;
		
			if(state==osUnknown)
			{
				state = osContinue;
				switch (Key)
				{
				case kRed:
					return ParseCommand(GetHelpCommand(0));
					break;
				case kGreen:
					return ParseCommand(GetHelpCommand(1));
					break;
				case kYellow:
					return ParseCommand(GetHelpCommand(2));
					break;
				case kBlue:
					return ParseCommand(GetHelpCommand(3));
					break;
				default:
					break;
				}
			}
		}
		break;
	default:
		state = cMenuText::ProcessKey(Key);
		if(hadSubmenu && !HasSubMenu())
		{
			if(GetFileName()) LoadContent(GetFileName());
			return osContinue;
		}
	}
	return state;
}


/* ########################################################
   ------------------- Men Refresh------------------------
   ######################################################## */

cSportNGRefresh::cSportNGRefresh(int Mode, const void* Var) : 
cOsdMenu("")
{
	if(Mode == REFRESH)
	{
		SetTitle(tr("Refresh Data"));
		if(Reload() == osBack)
			cRemote::Put(kBack, true);
	}
	else if(Mode == UPDATE)
	{
		SetTitle(cString::sprintf(tr("Update to %s"), (const char*)Var));
		if(Update((const char*)Var) == osBack)
			cRemote::Put(kBack, true);
	}
	
}
eOSState cSportNGRefresh::Reload()
{
	eOSState State = osContinue;
	Clear();
//	Skins.Flush();
	SetCols(30);
	Display();
	Skins.Flush();
	if(ExecCommand(tr("Checking Network"), cString::sprintf("'%s/%s' network", ScriptDir, "sportng.sh"), /*bool NiceLine=*/false))
		if(ExecCommand(tr("Downloading Data"), cString::sprintf("'%s/%s' download '%s' 'sportng=%s'", ScriptDir, "sportng.sh", DataDir, SportNG_VERSION)))
			State = osBack;
	if(State == osBack)
	{
		Completed(tr("R E F R E S H   C O M P L E T E D"));
		GetNextUpdate(true); // SportNG@NextUpdate neu einlesen
		cCondWait::SleepMs(800);
	}
	else
	{
		SetHelp(NULL,NULL,tr("Refresh"),tr("Back"));
	}
	return State;
}
eOSState cSportNGRefresh::Update(const char* File)
{
	eOSState State = osContinue;
	Clear();
	SetCols(30);
	Display();
	Skins.Flush();
	if(ExecCommand(tr("Checking Network"), cString::sprintf("'%s/%s' network", ScriptDir, "sportng.sh"), /*bool NiceLine=*/false))
		if(ExecCommand(tr("Downloading Data"), cString::sprintf("'%s/%s' update '%s'", ScriptDir, "sportng.sh", File)))
			State = osBack;
	if(State == osBack)
	{
		Completed(tr("D O W N L O A D   C O M P L E T E D"));
		cCondWait::SleepMs(800);
	}
	else
	{
		SetHelp(NULL,NULL,NULL,tr("Back"));
	}
	return State;
}
void cSportNGRefresh::Completed(const char *Message)
{
	Add(new cOsdItem("" , osUnknown, false));
	Add(new cOsdItem("" , osUnknown, false));
	cOsdItem *lastItem = new cOsdItem(Message, osUnknown, false);
	Add(lastItem);
	DISPLAYITEM(lastItem);
	Skins.Flush();
	for(int i=0; i<6; i++)
	{
		cCondWait::SleepMs(75);
		lastItem->SetText("");
		DISPLAYITEM(lastItem);
		Skins.Flush();
		cCondWait::SleepMs(100);
		lastItem->SetText(Message);
		DISPLAYITEM(lastItem);
		Skins.Flush();
	}
}

class MySystem : public cThread
{
private:
	char *cmd;
	int state;
public:
	MySystem(const char *Cmd) : cThread("SportNG")
	{
		cmd = strdup(Cmd);
		state=314159265;
		if(Start());
			while(state==314159265)	cCondWait::SleepMs(3);
	}
	~MySystem() { free(cmd); }
	int State() { return state; }
protected:
	void Action(void)
	{
		state = -1;
		isyslog("%s: SystemExec(\"%s\");", PLUGIN_NAME_I18N, cmd);
		int Status = SystemExec(cmd);
		if(WIFEXITED(Status))
			state = WEXITSTATUS(Status);
		isyslog("%s: SystemExec(\"%s\")=%i;", PLUGIN_NAME_I18N, cmd, state);
	}
};
bool cSportNGRefresh::ExecCommand(const char *Message, const char *Command, bool NiceLine/*=true*/)
{
	cOsdItem *lastItem;
	int res=0;
	if(NiceLine)
	{
		lastItem = new cOsdItem("-------------------------------------------------------------------------------------------------", osUnknown, false);
		Add(lastItem);
		DISPLAYITEM(lastItem);
		Skins.Flush();
	}
	lastItem = new cOsdItem(cString::sprintf("%s\t.....", Message), osUnknown, false);
	Add(lastItem);
	DISPLAYITEM(lastItem);
	Skins.Flush();
	if(Command)
	{
		int i=0;
		int d=1;
		const char *Messages[5] = {"O....",".O...","..O..","...O.","....O"};
		MySystem System(Command);
		while(System.Active())
		{
			lastItem->SetText(cString::sprintf("%s\t%s", Message, Messages[i]));
			DISPLAYITEM(lastItem);
			Skins.Flush();
			i += d;
			if(i==0 || i==4)	d = -d;
			cCondWait::SleepMs(80);
		}
		res = System.State();
//		res = system(Command);
		if(res)
			lastItem->SetText(cString::sprintf("%s\t%s", Message, tr("Failed")));
		else
			lastItem->SetText(cString::sprintf("%s\t%s", Message, tr("OK")));
		DISPLAYITEM(lastItem);
		Skins.Flush();
	}
	return !res;
}
eOSState cSportNGRefresh::ProcessKey(eKeys Key)
{
	bool hadSubmenu = HasSubMenu();
	eOSState state = cOsdMenu::ProcessKey(Key);
	if(hadSubmenu && !HasSubMenu())
		return osContinue;
	if(state==osUnknown)
	{
		state = osContinue;
		switch (Key)
		{
		case kYellow:
			return Reload();
			break;
		case kBlue:
		case kOk:
			return osBack;
			break;
		default:
			;
		}
	}
	return state;
}

